@@ -23,8 +23,6 @@ NUMBER_START = re.compile(r"^(\s*-?[0-9]+)")
 
 NON_WORD_START = re.compile(r"^\W+")
 
-NON_WORD_ALL = re.compile(r"^\W+$")
-
 
 class HassilError(Exception):
     """Base class for hassil errors"""
@@ -48,6 +46,9 @@ class MatchEntity:
     value: Any
     """Value of the entity."""
 
+    text: str
+    """Original value text."""
+
 
 @dataclass
 class MatchContext:
@@ -91,6 +92,9 @@ class RecognizeResult:
     entities_list: List[MatchEntity] = field(default_factory=list)
     """Matched entities as a list (duplicates allowed)."""
 
+    response: Optional[str] = None
+    """Key for intent response."""
+
 
 def recognize(
     text: str,
@@ -99,6 +103,7 @@ def recognize(
     expansion_rules: Optional[Dict[str, Sentence]] = None,
     skip_words: Optional[Iterable[str]] = None,
     intent_context: Optional[Dict[str, Any]] = None,
+    default_response: Optional[str] = "default",
 ) -> Optional[RecognizeResult]:
     """Return the first match of input text/words against a collection of intents."""
     text = normalize_text(text)
@@ -208,17 +213,22 @@ def recognize(
                         # Add fixed entities
                         for slot_name, slot_value in intent_data.slots.items():
                             maybe_match_context.entities.append(
-                                MatchEntity(name=slot_name, value=slot_value)
+                                MatchEntity(name=slot_name, value=slot_value, text="")
                             )
 
                         # Return the first match
+                        response = default_response
+                        if intent_data.response is not None:
+                            response = intent_data.response
+
                         return RecognizeResult(
-                            intent,
-                            {
+                            intent=intent,
+                            entities={
                                 entity.name: entity
                                 for entity in maybe_match_context.entities
                             },
-                            maybe_match_context.entities,
+                            entities_list=maybe_match_context.entities,
+                            response=response,
                         )
 
     return None
@@ -265,7 +275,11 @@ def is_match(
 
 
 def _remove_skip_words(text: str, skip_words: Iterable[str]) -> str:
-    for skip_word in skip_words:
+    """Remove skip words from text."""
+
+    # It's critical that skip words are processed longest first, since they may
+    # share prefixes.
+    for skip_word in sorted(skip_words, key=len, reverse=True):
         skip_word = normalize_text(skip_word)
         text = re.sub(rf"\b{re.escape(skip_word)}\b", "", text)
 
@@ -283,7 +297,7 @@ def match_expression(
         chunk: TextChunk = expression
         chunk_text = chunk.text.lstrip()
 
-        if (not context.text.strip()) or chunk.is_empty:
+        if chunk.is_empty:
             # Skip empty chunk
             yield context
         elif context.text.startswith(chunk_text):
@@ -305,12 +319,12 @@ def match_expression(
         seq: Sequence = expression
         if seq.type == SequenceType.ALTERNATIVE:
             # Any may match (words | in | alternative)
+            # NOTE: [optional] = (optional | )
             for item in seq.items:
                 yield from match_expression(context, item)
 
         elif seq.type == SequenceType.GROUP:
             # All must match (words in group)
-            # NOTE: [optional] = (optional | )
             if seq.items:
                 group_contexts = [context]
                 for item in seq.items:
@@ -346,17 +360,26 @@ def match_expression(
                     )
 
                     for value_context in value_contexts:
-                        value_context.entities.append(
+                        entities = context.entities + [
                             MatchEntity(
-                                name=list_ref.slot_name, value=slot_value.value_out
+                                name=list_ref.slot_name,
+                                value=slot_value.value_out,
+                                text=context.text[: -len(value_context.text)],
                             )
-                        )
+                        ]
 
                         if slot_value.context:
                             # Merge context from matched list value
-                            value_context.intent_context.update(slot_value.context)
-
-                        yield value_context
+                            yield dataclasses.replace(
+                                value_context,
+                                entities=entities,
+                                intent_context={
+                                    **context.intent_context,
+                                    **slot_value.context,
+                                },
+                            )
+                        else:
+                            yield dataclasses.replace(value_context, entities=entities)
 
         elif isinstance(slot_list, RangeSlotList):
             # List that represents a number range.
@@ -377,12 +400,18 @@ def match_expression(
                         )
 
                     if in_range:
-                        context.entities.append(
-                            MatchEntity(name=list_ref.slot_name, value=word_number)
-                        )
+                        entities = context.entities + [
+                            MatchEntity(
+                                name=list_ref.slot_name,
+                                value=word_number,
+                                text=context.text.split()[0],
+                            )
+                        ]
 
                         yield dataclasses.replace(
-                            context, text=context.text[len(number_text) :].lstrip()
+                            context,
+                            text=context.text[len(number_text) :].lstrip(),
+                            entities=entities,
                         )
 
         else:
