@@ -11,7 +11,9 @@ from hassil import (
 
 
 def test_just_words():
-    assert parse_sentences(["this is a test"]) == [s([tc("this is a test")])]
+    assert parse_sentences(["this is a test"]) == [
+        s([tc("this "), tc("is "), tc("a "), tc("test")])
+    ]
 
 
 def test_group():
@@ -19,7 +21,7 @@ def test_group():
         s(
             [
                 grp(
-                    [tc("this is a test")],
+                    [tc("this "), tc("is "), tc("a "), tc("test")],
                 )
             ]
         )
@@ -30,7 +32,8 @@ def test_optional():
     assert parse_sentences(["this is [a] test"]) == [
         s(
             [
-                tc("this is "),
+                tc("this "),
+                tc("is "),
                 alt(
                     [tc("a"), TextChunk.empty()],
                 ),
@@ -40,6 +43,22 @@ def test_optional():
     ]
 
 
+def test_optional_in_word():
+    assert parse_sentences(["turn on the light[s]"]) == [
+        s(
+            [
+                tc("turn "),
+                tc("on "),
+                tc("the "),
+                tc("light"),
+                alt(
+                    [tc("s"), TextChunk.empty()],
+                ),
+            ]
+        )
+    ]
+
+
 def test_optional_nested():
     assert parse_sentences(["this [is [a]] test"]) == [
         s(
@@ -61,9 +80,10 @@ def test_alternative():
     assert parse_sentences(["this is (a | the) test"]) == [
         s(
             [
-                tc("this is "),
+                tc("this "),
+                tc("is "),
                 alt(
-                    [grp([tc("a ")]), grp([tc("the")])],
+                    [grp([tc("a")]), grp([tc("the")])],
                 ),
                 tc(" test"),
             ]
@@ -75,14 +95,15 @@ def test_alternative_multiple_words():
     assert parse_sentences(["this is (a bigger | the biggest) test"]) == [
         s(
             [
-                tc("this is "),
+                tc("this "),
+                tc("is "),
                 alt(
                     [
                         grp(
-                            [tc("a bigger ")],
+                            [tc("a "), tc("bigger")],
                         ),
                         grp(
-                            [tc("the biggest")],
+                            [tc("the "), tc("biggest")],
                         ),
                     ],
                 ),
@@ -93,22 +114,22 @@ def test_alternative_multiple_words():
 
 
 def test_alternative_what():
-    assert parse_sentences(["(what | what's | whats | what is)"]) == [
+    assert parse_sentences(["( what | what's | whats | what is )"]) == [
         s(
             [
                 alt(
                     [
                         grp(
-                            [tc("what ")],
+                            [tc("what")],
                         ),
                         grp(
-                            [tc("what's ")],
+                            [tc("what's")],
                         ),
                         grp(
-                            [tc("whats ")],
+                            [tc("whats")],
                         ),
                         grp(
-                            [tc("what is")],
+                            [tc("what "), tc("is")],
                         ),
                     ],
                 ),
@@ -119,25 +140,34 @@ def test_alternative_what():
 
 def test_list_reference():
     assert parse_sentences(["this is a {test}"]) == [
-        s([tc("this is a "), ListReference("test")])
+        s([tc("this "), tc("is "), tc("a "), ListReference("test")])
     ]
 
 
 def test_list_reference_prefix_suffix():
     assert parse_sentences(["this is a pre'{test}-post"]) == [
-        s([tc("this is a pre'"), ListReference("test"), tc("-post")])
+        s(
+            [
+                tc("this "),
+                tc("is "),
+                tc("a "),
+                tc("pre'"),
+                ListReference("test"),
+                tc("-post"),
+            ]
+        )
     ]
 
 
 def test_rule_reference():
     assert parse_sentences(["this is a <test>"]) == [
-        s([tc("this is a "), RuleReference("test")])
+        s([tc("this "), tc("is "), tc("a "), RuleReference("test")])
     ]
 
 
 def test_escape():
     assert parse_sentences(["this \\[is\\] a \\{test\\}"]) == [
-        s([tc("this [is] a {test}")])
+        s([tc("this "), tc("[is] "), tc("a "), tc("{test}")])
     ]
 
 
